/* eslint-disable consistent-return */
import { Server } from '..';
import { Route } from '../../class';
import { Req } from '../interfaces';

export default class Account extends Route {
  constructor(server: Server) {
    super(server, { path: '/account', deprecated: false, maintenance: false });
  }

  public bind() {
    this.router.use(async (req, res, next) => {
      await this.authorize(req, res, next);
    });

    this.router.get('/', async (req: Req, res) => {
      try {
        const acc: any = {};
        acc.username = req.account.username;
        acc.userID = req.account.userID;
        acc.email = req.account.emailAddress;
        acc.locked = req.account.locked;
        acc.root = req.account.root;
        acc.createdAt = req.account.createdAt;
        acc.createdBy = req.account.createdBy;
        acc.permissions = req.account.permissions;
        res.status(200).json({ code: this.constants.codes.SUCCESS, message: acc });
      } catch (error) {
        this.handleError(error, res);
      }
    });

    this.router.get('/moderations/:id?', async (req: Req, res) => {
      try {
        const moderations = await this.server.client.db.Moderation.find({ username: req.account.username });
        if (!moderations.length) return res.status(204).json({ code: this.constants.codes.NOT_FOUND, message: null });
        if (req.params.id) {
          const filtered = moderations.filter((moderation) => moderation.logID === req.params.id);
          res.status(200).json({ code: this.constants.codes.SUCCESS, message: { filtered } });
        } else {
          res.status(200).json({ code: this.constants.codes.SUCCESS, message: moderations });
        }
      } catch (error) {
        this.handleError(error, res);
      }
    });

    this.router.get('/resources', async (req: Req, res) => {
      try {
        const [cpuUsage, ramUsage, diskUsage] = await Promise.all([
          this.server.client.util.exec(`top -b -n 1 -u ${req.account.username} | awk 'NR>7 { sum += $9; } END { print sum; }'`),
          this.server.client.util.exec(`memory ${req.account.username}`),
          this.server.client.redis.get(`storage-${req.account.username}`),
        ]);
        let storage: string;
        if (req.query?.cache === 'false') {
          const val = await this.server.client.util.exec(`du -sb /home/${req.account.username}`);
          // eslint-disable-next-line prefer-destructuring
          storage = val.split('\t')[0];
        } else {
          storage = diskUsage;
        }
        res.status(200).json({ code: this.constants.codes.SUCCESS,
          message: {
            cpu: Number(`${cpuUsage.split('\n')[0] || '0'}`),
            ram: (Number(ramUsage) * 1000) / 1024 / 1024,
            disk: Number(storage) / 1024 / 1024,
          },
        });
      } catch (error) {
        this.handleError(error, res);
      }
    });
  }
}
