/* eslint-disable import/no-unresolved */
/* eslint-disable no-await-in-loop */
/* eslint-disable no-param-reassign */
import axios from 'axios';
import { randomBytes } from 'crypto';
import childProcess from 'child_process';
import nodemailer from 'nodemailer';
import { ColorResolvable, DMChannel, Message, MessageEmbed, TextChannel, User } from 'discord.js';
import { v4 as uuid } from 'uuid';
import moment from 'moment';
import fs from 'fs';
import hastebin from 'hastebin-gen';
import { getUserByUid } from '../functions';
import { AccountUtil, Client, Command, PaginationEmbed } from '.';
import { AccountInterface, ModerationInterface } from '../models';
import { Certificate } from '../../types/x509';

enum TechnicianNameFormatOpt {
  Full = 0,
  Partial,
  Basic
}

export default class Util {
  public client: Client;

  public accounts: AccountUtil;

  public transport: nodemailer.Transporter;

  constructor(client: Client) {
    this.client = client;
    this.transport = nodemailer.createTransport({
      host: 'staff.libraryofcode.org',
      auth: { user: 'support', pass: this.client.config.emailPass },
    });
    this.accounts = new AccountUtil(client);
  }

  /**
   * Executes a terminal command async.
   * @param command The command to execute
   * @param options childProcess.ExecOptions
   */
  public async exec(command: string, options: childProcess.ExecOptions = {}): Promise<string> {
    return new Promise((res, rej) => {
      let output = '';
      const writeFunction = (data: string|Buffer|Error) => {
        output += `${data}`;
      };
      const cmd = childProcess.exec(command, options);
      cmd.stdout.on('data', writeFunction);
      cmd.stderr.on('data', writeFunction);
      cmd.on('error', writeFunction);
      cmd.once('close', (code) => {
        cmd.stdout.off('data', writeFunction);
        cmd.stderr.off('data', writeFunction);
        cmd.off('error', writeFunction);
        setTimeout(() => {}, 1000);
        if (code !== 0) rej(new Error(`Command failed: ${command}\n${output}`));
        res(output);
      });
    });
    /* return new Promise((resolve, reject) => {
      childProcess.exec(command, options, (err, stdout, stderr) => {
        if (stderr) reject(new Error(`Command failed: ${command}\n${stderr}`));
        if (err) reject(err);
        resolve(stdout);
      });
    }); */
  }

  public sanitize(str: string): string {
    let ret = str;
    ret = ret.replace('\\', '~');
    ret = ret.replace('.', '~');
    ret = ret.replace(/\W|_/g, '~');
    return ret;
  }

  public async sendMessageToUserTerminal(username: string, message: string): Promise<boolean> {
    const ptsArray = await this.getPTS(username);
    if (!ptsArray) return false;
    for (const pts of ptsArray) {
      const msg = `\n==SYSTEM NOTIFICATION | CLOUD SERVICES MANAGEMENT DAEMON==\n${new Date().toLocaleString('en-us')}\n\n${message}\n`;
      await this.exec(`echo "${msg}" >> /dev/pts/${pts}`);
    }
    return true;
  }

  private async getPTS(username: string): Promise<number[] | undefined> {
    const dir = await fs.promises.readdir('/dev/pts');
    const returnArray: number[] = [];
    for (const file of dir) {
      const fileInformation = await fs.promises.stat(`/dev/pts/${file}`);
      const user = await getUserByUid(this.client, fileInformation.uid);
      if (user && user === username) {
        returnArray.push(Number(file));
      }
    }
    if (returnArray.length < 1) return undefined;
    return returnArray;
  }

  /**
   * Resolves a command
   * @param query Command input
   * @param message Only used to check for errors
   */
  public resolveCommand(query: string | string[], message?: Message): Promise<{cmd: Command, args: string[] }> {
    try {
      let resolvedCommand: Command;
      if (typeof query === 'string') query = query.split(' ');
      const commands = this.client.commands.toArray();
      resolvedCommand = commands.find((c) => c.name === query[0].toLowerCase() || c.aliases.includes(query[0].toLowerCase()));

      if (!resolvedCommand) return Promise.resolve(null);
      query.shift();
      while (resolvedCommand.subcommands.size && query.length) {
        const subCommands = resolvedCommand.subcommands.toArray();
        const found = subCommands.find((c) => c.name === query[0].toLowerCase() || c.aliases.includes(query[0].toLowerCase()));
        if (!found) break;
        resolvedCommand = found;
        query.shift();
      }
      return Promise.resolve({ cmd: resolvedCommand, args: query });
    } catch (error) {
      if (message) this.handleError(error, message);
      else this.handleError(error);
      return Promise.reject(error);
    }
  }

  public async handleError(error: Error | any, message?: Message, command?: Command): Promise<void> {
    try {
      this.client.signale.error(error);
      const info = { content: `\`\`\`js\n${error.stack}\n\`\`\``, embed: null };
      if (message) {
        const embed = new MessageEmbed();
        embed.setColor('#FF0000');
        embed.setAuthor(`Error caused by ${message.author.username}#${message.author.discriminator}`, message.author.avatarURL());
        embed.setTitle('Message content');
        embed.setDescription(message.content);
        embed.addField('User', `${message.author.toString()} (\`${message.author.id}\`)`, true);
        embed.addField('Channel', `<#${message.channel.id}>`, true);
        let guild: string;
        if (message.channel instanceof DMChannel) guild = '@me';
        else guild = message.guild.id;
        embed.addField('Message link', `[Click here](https://discordapp.com/channels/${guild}/${message.channel.id}/${message.id})`, true);
        embed.setTimestamp(new Date(message.createdTimestamp));
        info.embed = embed;
      }
      const ch = this.client.channels.cache.get('595788220764127272') as TextChannel;
      await ch.send(info);
      const msg = message.content.slice(this.client.config.prefix.length).trim().split(/ +/g);
      if (command) this.resolveCommand(msg).then((c) => { c.cmd.enabled = false; });
      if (message) message.channel.send(`***${this.client.stores.emojis.error} An unexpected error has occurred - please contact a member of the Engineering Team.${command ? ' This command has been disabled.' : ''}***`);
    } catch (err) {
      this.client.signale.error(err);
    }
  }

  public createPaginationEmbed = PaginationEmbed;

  public splitFields(fields: { name: string, value: string, inline?: boolean }[]): { name: string, value: string, inline?: boolean }[][] {
    let index = 0;
    const array: {name: string, value: string, inline?: boolean}[][] = [[]];
    while (fields.length) {
      if (array[index].length >= 25) { index += 1; array[index] = []; }
      array[index].push(fields[0]); fields.shift();
    }
    return array;
  }

  public splitString(string: string, length: number): string[] {
    if (!string) return [];
    if (Array.isArray(string)) string = string.join('\n');
    if (string.length <= length) return [string];
    const arrayString: string[] = [];
    let str: string = '';
    let pos: number;
    while (string.length > 0) {
      pos = string.length > length ? string.lastIndexOf('\n', length) : string.length;
      if (pos > length) pos = length;
      str = string.substr(0, pos);
      string = string.substr(pos);
      arrayString.push(str);
    }
    return arrayString;
  }

  public async createHash(password: string): Promise<string> {
    return this.exec(`mkpasswd -m sha-512 "${password}"`);
  }

  public isValidEmail(email: string): boolean {
    const checkAt = email.indexOf('@');
    if (checkAt < 1) return false;
    const checkDomain = email.indexOf('.', checkAt + 2);
    return checkDomain >= checkAt;
  }

  public randomPassword(): string {
    let tempPass = ''; const passChars = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0'];
    while (tempPass.length < 5) { tempPass += passChars[Math.floor(Math.random() * passChars.length)]; }
    return tempPass;
  }

  public async createAccount(hash: string, etcPasswd: string, username: string, userID: string, emailAddress: string, moderatorID: string, code: string): Promise<AccountInterface> {
    await this.exec(`useradd -m -p ${hash} -c ${etcPasswd} -s /bin/bash ${username}`);
    await this.exec(`chage -d0 ${username}`);
    const tier = await this.client.db.Tier.findOne({ id: 1 });

    const account = new this.client.db.Account({
      username, userID, emailAddress, createdBy: moderatorID, createdAt: new Date(), locked: false, tier: 1, supportKey: code, totalReferrals: 0, referralCode: randomBytes(9).toString('base64').toUpperCase(), ssInit: false, ramLimitNotification: tier.resourceLimits.ram - 50, homepath: `/home/${username}`,
    });
    return account.save();
  }

  public async deleteAccount(username: string): Promise<void> {
    const account = await this.client.db.Account.findOne({ username });
    if (!account) throw new Error('Account not found');
    this.exec(`lock ${username}`);
    const tasks = [
      this.exec(`deluser ${username} --remove-home --backup-to /management/Archives && rm -rf -R ${account.homepath}`),
      this.client.db.Account.deleteOne({ username }),
    ];
    const guild = this.client.guilds.cache.get('446067825673633794');
    const member = await guild.members.fetch(account.userID);
    await member.roles.remove('546457886440685578', 'Cloud Account Deleted');
    // @ts-ignore
    await Promise.all(tasks);
  }

  public async messageCollector(message: Message, question: string, timeout: number, shouldDelete = false, choices: string[] = null, filter = (msg: Message): boolean|void => {}): Promise<Message> {
    const msg = await message.channel.send(question);
    return new Promise((res, rej) => {
      const func = (Msg: Message) => {
        if (filter(Msg) === false) return;
        const verif = choices ? choices.includes(Msg.content) : Msg.content;
        if (verif) { if (shouldDelete) msg.delete().catch(); res(Msg); }
      };

      setTimeout(() => {
        if (shouldDelete) msg.delete().catch(); rej(new Error('Did not supply a valid input in time'));
        this.client.removeListener('messageCreate', func);
      }, timeout);
      this.client.on('messageCreate', func);
    });
  }

  /**
   * @param type `0` - Create
   *
   * `1` - Warn
   *
   * `2` - Lock
   *
   * `3` - Unlock
   *
   * `4` - Delete
   */
  public async createModerationLog(user: string, moderator: User, type: number, reason?: string, duration?: number): Promise<ModerationInterface> {
    const moderatorID = moderator.id;
    const account = await this.client.db.Account.findOne({ $or: [{ username: user }, { userID: user }] });
    if (!account) return Promise.reject(new Error(`Account ${user} not found`));
    const { username, userID } = account;
    const logInput: { username: string, userID: string, logID: string, moderatorID: string, reason?: string, type: number, date: Date, expiration?: { date: Date, processed: boolean }} = {
      username, userID, logID: uuid(), moderatorID, type, date: new Date(),
    };

    const now: number = Date.now();
    let date: Date;
    let processed = true;
    if (reason) logInput.reason = reason;
    if (type === 2) {
      if (duration) {
        date = new Date(now + duration);
        processed = false;
      } else date = null;
    }

    const expiration: { date: Date; processed: boolean } = { date, processed };

    logInput.expiration = expiration;
    const log = new this.client.db.Moderation(logInput);
    await log.save();

    let embedTitle: string;
    let color: ColorResolvable;
    let archType: string;
    switch (type) {
      default: archType = 'Staff'; embedTitle = 'Cloud Account | Generic'; color = '#0892e1'; break;
      case 0: archType = 'Technician'; embedTitle = 'Cloud Account | Create'; color = '#00ff00'; break;
      case 1: archType = 'Technician'; embedTitle = 'Account Warning | Warn'; color = '#ffff00'; break;
      case 2: archType = 'Technician'; embedTitle = 'Account Infraction | Lock'; color = '#ff6600'; break;
      case 3: archType = 'Technician'; embedTitle = 'Account Infraction | Unlock'; color = '#0099ff'; break;
      case 4: archType = 'Manager'; embedTitle = 'Cloud Account | Delete'; color = '#ff0000'; break;
    }
    const req = await axios.get('https://loc.sh/int/directory');
    const find = req.data.find((mem) => mem.userID === moderator.id);
    const embed = new MessageEmbed()
      .setTitle(embedTitle)
      .setColor(color)
      .addField('User', `${username} | <@${userID}>`, true)
      .addField(archType, moderatorID === this.client.user.id ? 'SYSTEM' : `${moderator.username}${find.isManager ? ' [k] ' : ' '}(<@${moderatorID}>)`, true)
      .setFooter(this.client.user.username, this.client.user.avatarURL())
      .setTimestamp();
    if (reason) embed.addField('Reason', reason || 'Not specified');
    if (type === 2) embed.addField('Lock Expiration', `${date ? moment(date).format('dddd, MMMM Do YYYY, h:mm:ss A') : 'Indefinitely'}`);
    const ch = this.client.channels.cache.get('580950455581147146') as TextChannel;
    ch.send({ embeds: [embed] });
    this.client.users.fetch(userID).then((channel) => channel.send({ embeds: [embed] })).catch();

    return Promise.resolve(log);
  }

  public async getTechnicianName(tech: User, format: TechnicianNameFormatOpt = TechnicianNameFormatOpt.Full) {
    if (!tech) return 'SYSTEM (UT)';
    if (tech.id === this.client.user.id) return 'SYSTEM (SELF)';

    const req = await axios.get('https://loc.sh/int/directory');
    const find = req.data.find((mem) => mem.userID === tech.id);
    if (!find) return 'SYSTEM (UF)';
    if (format === TechnicianNameFormatOpt.Full) {
      return `${tech.username}${find.isManager ? ' [k]' : ''}${find.title ? ` (${find.title} / ${find.dept})` : ` (${find.dept})`} <<@${tech.id}>>`;
    } else if (format === TechnicianNameFormatOpt.Partial) {
      return `${tech.username}${find.isManager ? ' [k]' : ''}${find.title ? ` (${find.title})` : ` (${find.dept})`} <<@${tech.id}>>`;
    }
    return `${tech.username}${find.isManager ? ' [k]' : ''} <<@${tech.id}>>`;
  }

  public parseCertificate(pem: string) {
    return axios.post<Certificate>('https://certapi.libraryofcode.org/parse', pem)
      .then((response) => response.data);
  }

  public upload(text: string, extension = 'txt') {
    return hastebin(text, {
      url: 'https://snippets.cloud.libraryofcode.org',
      extension,
    });
  }
}
