import { Message } from 'discord.js';
import { Client, Command } from '../class';

export default class Bearer_Revoke extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'revoke';
    this.description = 'Revokes an API bearer token.';
    this.usage = `${this.client.config.prefix}bearer revoke <token>`;
    this.enabled = true;
    this.guildOnly = false;
  }

  public async run(message: Message, args: string[]) {
    try {
      message.delete();
      if (!args[0]) return this.client.commands.get('help').run(message, ['bearer', this.name]);
      const account = await this.client.db.Account.findOne({ userID: message.author.id });
      if (!account) return this.error(message.channel, 'You do not have an account.');

      const bearerVerify = await this.client.server.security.checkBearer(args[0]);
      if (!bearerVerify || bearerVerify?.userID !== account.userID) return this.error(message.channel, 'Permission denied.');
      if (account.revokedBearers?.includes(args[0])) return this.error(message.channel, 'This bearer token is already revoked.');
      await account.updateOne({ $addToSet: { revokedBearers: args[0] } });
      return this.success(message.channel, 'Revoked bearer token.');
    } catch (err) {
      return this.client.util.handleError(err, message, this);
    }
  }
}
