import moment, { unitOfTime } from 'moment';
import { Message } from 'discord.js';
import { Client, Command } from '../class';

export default class Lock extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'lock';
    this.description = 'Locks an account.';
    this.permissions = { roles: ['662163685439045632', '701454780828221450'] };
    this.enabled = true;
    this.usage = `${this.client.config.prefix}lock [User Name | User ID/Mention] <Time> [Reason]`;
  }

  public async run(message: Message, args: string[]) {
    try {
      if (!args.length) return this.client.commands.get('help').run(message, [this.name]);
      const account = await this.client.db.Account.findOne({ $or: [{ username: args[0] }, { userID: args[0].replace(/[<@!>]/gi, '') }] });
      if (!account) return this.error(message.channel, 'Cannot find user.');
      if (account.locked) return this.error(message.channel, 'This account is already locked.');
      const edit = await this.loading(message.channel, 'Locking account...');
      if (account.username === 'matthew' || account.root) return edit.edit(`***${this.client.stores.emojis.error} Permission denied.***`);
      await this.client.util.sendMessageToUserTerminal(account.username, 'ACCOUNT LOCKED BY TECHNICIAN | PREPARING TO LOGOUT').catch(() => { });
      await this.client.util.exec(`lock ${account.username}`);
      await account.updateOne({ locked: true });

      const expiry = new Date();
      const [length, unit] = args[1].match(/[a-z]+|[^a-z]+/gi);
      const momentMilliseconds = moment.duration(Number(length), unit as unitOfTime.Base).asMilliseconds();
      const reason = momentMilliseconds ? args.slice(2).join(' ') : args.slice(1).join(' ');

      await this.client.util.createModerationLog(account.userID, message.author, 2, reason, momentMilliseconds);
      edit.edit(`***${this.client.stores.emojis.success} Account ${account.username} has been locked by Technician ${message.author.username}#${message.author.discriminator}.***`);
      message.delete();

      this.client.util.transport.sendMail({
        to: account.emailAddress,
        from: 'Library of Code Inc | Cloud Services <help@libraryofcode.org>',
        replyTo: 'cloud-help@libraryofcode.org',
        subject: 'Your account has been locked',
        html: `
        <h1>Library of Code | Cloud Services</h1>
        <p>Your Cloud Account has been locked until ${momentMilliseconds ? moment(expiry).calendar() : 'indefinitely'} under the EULA.</p>
        <p><b>Reason:</b> ${momentMilliseconds ? args.slice(2).join(' ') : args.slice(1).join(' ')}</p>
        <p><b>Technician:</b> ${message.author.username}</p>
        <p><b>Expiration:</b> ${momentMilliseconds ? moment(expiry).format('dddd, MMMM Do YYYY, h:mm:ss A') : 'N/A'}</p>

        <b><i>Library of Code Inc | Support Team</i></b>
        `,
      });
    } catch (error) {
      return this.client.util.handleError(error, message, this);
    }
  }
}
