import { Message } from 'discord.js';
import axios from 'axios';
import { Client, Command } from '../class';

export default class Pull extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'pull';
    this.description = 'Fetches the latest commit from Gitlab';
    this.aliases = ['update'];
    this.enabled = true;
    this.permissions = { roles: ['662163685439045632'] };
  }

  public async run(message: Message, args: string[]) {
    try {
      if (this.client.updating) return this.error(message.channel, 'An update is already in progress.');
      this.client.updating = true;
      const updateMessage = await this.loading(message.channel, 'Fetching latest commit...');
      let pull: string;

      try {
        pull = await this.client.util.exec('git pull', { cwd: '/opt/CloudServices' });
      } catch (error) {
        const updatedMessage = updateMessage.content.replace(`***${this.client.stores.emojis.loading} Fetching latest commit...***`, `***${this.client.stores.emojis.error} Could not fetch latest commit***\n`
          + `\`\`\`\n${error.message}\n\`\`\``);
        this.client.updating = false;
        return updateMessage.edit(updatedMessage);
      }
      if (pull.includes('Already up to date') && args[0] !== '-f') {
        const updatedMessage = updateMessage.content.replace(`***${this.client.stores.emojis.loading} Fetching latest commit...***`, `***${this.client.stores.emojis.success} No updates available***`);
        this.client.updating = false;
        return updateMessage.edit(updatedMessage);
      }
      if (!pull.includes('origin/master') && !pull.includes(' changed, ')) {
        const updatedMessage = updateMessage.content.replace(`***${this.client.stores.emojis.loading} Fetching latest commit...***`, `***${this.client.stores.emojis.error} Unexpected git output***`);
        this.client.updating = false;
        return updateMessage.edit(updatedMessage);
      }
      const continueMessage = updateMessage.content.replace(`***${this.client.stores.emojis.loading} Fetching latest commit...***`, `***${this.client.stores.emojis.success} ${pull.includes('Already up to date') ? 'No updates available' : 'Fetched latest commit'}***\n`
        + `***${this.client.stores.emojis.loading} Reinstalling dependencies...***`);
      const passedPull = await updateMessage.edit(continueMessage);

      let install: string;
      try {
        install = await this.client.util.exec('yarn install', { cwd: '/opt/CloudServices' });
      } catch (error) {
        this.client.updating = false;
        const updatedMessage = passedPull.content.replace(`***${this.client.stores.emojis.loading} Reinstalling dependencies...***`, `***${this.client.stores.emojis.error} Failed to reinstall dependencies***\n`
          + `\`\`\`\n${error.message}\n\`\`\``);
        return updateMessage.edit(updatedMessage);
      }
      let updatedPackages: Message;
      if (install.includes('Already up-to-date')) {
        const updatedMessage = passedPull.content.replace(`***${this.client.stores.emojis.loading} Reinstalling dependencies...***`, `***${this.client.stores.emojis.success} No dependency updates available***\n***${this.client.stores.emojis.loading} Rebuilding files...***`);
        updatedPackages = await updateMessage.edit(updatedMessage);
      } else {
        const updatedMessage = passedPull.content.replace(`***${this.client.stores.emojis.loading} Reinstalling dependencies...***`, `***${this.client.stores.emojis.success} Updated dependencies***\n***${this.client.stores.emojis.loading} Rebuilding files...***`);
        updatedPackages = await updateMessage.edit(updatedMessage);
      }

      let build: string;
      try {
        build = await this.client.util.exec('yarn run build', { cwd: '/opt/CloudServices' });
      } catch (error) {
        let updatedMessage = updatedPackages.content.replace(`***${this.client.stores.emojis.loading} Rebuilding files...***`, `***${this.client.stores.emojis.error} Failed to rebuild files***\n`);
        if (error.message.length < 1800) updatedMessage += `\`\`\`\n${error.message}\n\`\`\``;
        else {
          const split = this.client.util.splitString(error.message, 1975);
          if (split[5]) {
            try {
              const { data } = await axios.post('https://snippets.cloud.libraryofcode.org/documents', split.join(''));
              updatedMessage += `${this.client.stores.emojis.success} The error output can be found on https://snippets.cloud.libraryofcode.org/${data.key}`;
            } catch (e) {
              updatedMessage += `${this.client.stores.emojis.error} Could not upload error: ${e}`;
            }
          } else {
            split.forEach((m) => message.channel.send(`\`\`\`bash\n${m}\n\`\`\``));
          }
        }
        this.client.buildError = true;
        this.client.updating = false;
        return updateMessage.edit(updatedMessage);
      }
      const finalMessage = updatedPackages.content.replace(`***${this.client.stores.emojis.loading} Rebuilding files...***`, `***${this.client.stores.emojis.success} Files rebuilt***`);
      this.client.updating = false;
      this.client.buildError = false;
      return updateMessage.edit(finalMessage);
    } catch (error) {
      this.client.updating = false;
      return this.client.util.handleError(error, message, this);
    }
  }
}
