import { Message, TextChannel, MessageEmbed } from 'discord.js';
import { Client, Event } from '../class';

export default class extends Event {
    public client: Client

    constructor(client: Client) {
      super(client);
      this.client = client;
      this.event = 'messageCreate';
    }

    public async run(message: Message) {
      try {
        if (message.author.bot && message.author.id !== '554168666938277889') return;
        if (message.content.indexOf(this.client.config.prefix) !== 0) return;
        const noPrefix: string[] = message.content.slice(this.client.config.prefix.length).trim().split(/ +/g);
        const resolved = await this.client.util.resolveCommand(noPrefix, message);
        if (!resolved) return;
        if (resolved.cmd.guildOnly && !(message.channel instanceof TextChannel)) return;
        let hasUserPerms: boolean;
        if (resolved.cmd.permissions.users) {
          hasUserPerms = resolved.cmd.permissions.users.includes(message.author.id);
        }
        let hasRolePerms: boolean = false;
        if (resolved.cmd.permissions.roles) {
          for (const role of resolved.cmd.permissions.roles) {
            if (message.member && message.member.roles.cache.has(role)) {
              hasRolePerms = true;
              break;
            }
          }
        }
        if (!resolved.cmd.permissions.users && !resolved.cmd.permissions.roles) {
          hasUserPerms = true;
          hasRolePerms = true;
        }
        if (message.author.id === '554168666938277889') {
          hasUserPerms = true;
          hasRolePerms = true;
        }
        if (!hasRolePerms && !hasUserPerms) return;
        if (!resolved.cmd.enabled) {
          message.channel.send(`***${this.client.stores.emojis.error} This command has been disabled***`);
          return;
        }
        await resolved.cmd.run(message, resolved.args);
      } catch (error) {
        this.client.util.handleError(error, message);
      }
    }
}
