/* eslint-disable no-await-in-loop */
import axios, { AxiosError } from 'axios';
import cron from 'cron';
import { MessageEmbed, TextChannel } from 'discord.js';
import { Client } from '../class';

async function checkt2(auth: string, userID: string) {
  try {
    const { data } = await axios({
      method: 'get',
      url: `https://eds.libraryofcode.org/cs/t2rev/?userID=${userID}&auth=${auth}`,
    });

    return {
      status: 'SUCCESS',
      decision: data.decision,
      id: data.id,
      processedBy: data.processedBy,
      token: data.token,
    };
  } catch (err) {
    const error = <AxiosError>err;
    if (error.response?.status === 404 || error.response.status === 400 || error.response.status === 401) return { id: 'N/A', processedBy: 'N/A', status: 'CLIENT_ERROR', decision: 'PRE-DECLINED' };
    return { id: 'N/A', processedBy: 'N/A', status: 'SERVER_ERROR', decision: 'PRE-DECLINED' };
  }
}

export default async function t2checkCron(client: Client) {
  const checkTier2StatusJob = new cron.CronJob('13 21 * * *', async () => {
    const users = await client.db.Account.find({ tier: 2 }).lean();

    for (const user of users) {
      const check = await checkt2(client.config.internalKey, user.userID);
      const member = client.guilds.cache.get('446067825673633794').members.cache.get(user.userID);
      if (member.roles.cache.has('585600289747369987') || member.roles.cache.has('858049948401401866') || member.roles.cache.has('995554888836718612')) continue;
      if (check.decision === 'DECLINED') {
        const embed = new MessageEmbed();
        embed.setTitle('Cloud Account | Tier Change');
        embed.setColor('#0099ff');
        embed.addField('User', `${user.username} | <@${user.userID}>`, true);
        embed.addField('Technician', 'SYSTEM', true);
        embed.addField('Old Tier -> New Tier', `${user.tier} -> 1`, true);
        embed.setFooter(client.user.username, client.user.avatarURL());
        embed.setTimestamp();
        const ch = client.channels.cache.get('580950455581147146') as TextChannel;
        await client.db.Account.updateOne({ userID: user.userID }, { $set: { tier: 1 } });
        ch.send({ embeds: [embed] });
        return client.users.fetch(user.userID).then((u) => u.send({ embeds: [embed], content: `__**Tier 2 Account Review**__\n\nHello, <@${user.userID}>.\nWe periodically run automatic account review inquiries for users with Tier 2. This means we request an account review from EDS to determine if you're still eligible to keep Tier 2.\nUnfortunately, according to EDS' records, you are currently ineligible for Tier 2. Because of this, we're going to automatically revoke your Tier 2 status. You can re-apply to Tier 2 at any time if you feel that you meet the requirements. If you have any questions, or if you believe this may be a mistake, please contact our support team as soon as possible. Thank you.` })).catch();
      }
    }
  });

  checkTier2StatusJob.start();
}
